# Copyright (C) 2009 Adam Blackburn
#
# This file is a part of Wicd.
#
# Wicd is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import logging
logger = logging.getLogger(__name__)

from wicd.core import event, providers
from wicd import paths
from wicd.extra.configobj import ConfigObj
import os

class OrderedProviderHandler(providers.ProviderHandler):
    def __init__(self, servicename, must_order=False):
        super(OrderedProviderHandler, self).__init__(servicename)
        self.must_order = must_order
        self._loading_plugins = False
        # make sure to to rerun set_order if something changes
        event.add_callback(self.changed, '%s_provider_added' % servicename)
        event.add_callback(self.changed, '%s_provider_removed' % servicename)
        event.add_callback(self.changed, 'done-loading')
        # also hook on to start-loading-plugins and stop-loading-plugins
        event.add_callback(self._start_loading_plugins,
                           'start-loading-plugins')
        event.add_callback(self._done_loading_plugins,
                           'done-loading-plugins')
        # grab the config path
        config_path = os.path.join(paths.SYSTEM_CONF, '%s.conf' % servicename)
        # create a ConfigObj config file
        self.config = ConfigObj(config_path, unrepr=True)
        self.set_order(self.config.get('order', default=[]))

    def _start_loading_plugins(self, *args):
        self._loading_plugins = True

    def _done_loading_plugins(self, *args):
        self._loading_plugins = False 

    def changed(self, *args):
        self.set_order(self.config.get('order', default=[]), save=False)

    def set_order(self, order, save=True):
        # don't bother while we're loading plugins
        if self._loading_plugins: return
        # should we save this new order?
        if save:
            self.config['order'] = order
            self.config.write()

        # make it easy to find providers by name
        def find_by_name(name, listing):
            for provider in listing:
                if provider.name == name:
                    return provider

        # grab a list of the providers we currently have
        remaining = self.get_providers()

        # if there are any
        if remaining:
            # try and sort them
            self.ordered_providers = []
            # loop through the names in the order specified
            for name in order:
                # try and find the class with the correct name
                classx = find_by_name(name, remaining)
                if classx:
                    # if we found it, stick it in the list
                    self.ordered_providers.append(classx)
                    # and remove it from remaining
                    remaining.remove(classx)
                elif self.must_order:
                    # if we didn't find it and must_order is on
                    # then we need to stop
                    raise ValueError(
                        'must_order is True, '
                        'but not all providers could be found')
                else:
                    # if must_order isn't on, then we can continue
                    # but issue a warning
                    logger.warning('could not find provider %s',
                                   name)

            if remaining:
                # if there were left over providers, warn about them
                logger.warning('not all providers for %s were ordered',
                              self.servicename)
                # and then tack them on the end
                self.ordered_providers.extend(remaining)

            logger.debug('providers for %s ordered: %s',
                     self.servicename,
                     self.ordered_providers)

        
    def get_preferred_provider(self):
        # check to make sure they've been ordered
        # and that there are actually providers
        if hasattr(self, 'ordered_providers') and \
           len(self.ordered_providers) > 0:
            # if so, return the first one
            return self.ordered_providers[0]
        elif len(providers) == 0:
            # no providers
            logger.warning('no providers available')
            # return None
            return None
        else:
            # providers haven't been ordered
            if self.must_order:
                raise ValueError(
                    'must_order is True, but providers are not ordered'
                )

            # get a list of the providers
            providers = self.get_providers()
            # warn that they haven't been sorted
            logger.warning('returned random provider %s', providers[0])
            # return the first one (it's random)
            return providers[0]
